// Trox 2.1.1 for Linux
// last change 1999-1-2
// Robin Gruber
// 2 to 4 players on up to 4 computers
 
#include "troxengine.h"
#include "troxintro.h"
#include "troxnet.h"
#include "troxmath.h"
#include "troxgyro.h"
#include "troxtimer.h"

#define DEBUG(x) 
// use DEBUG (x) x if you want some debugging info

// one cycle will be 30 ms
#define DELAY 30

DEBUG(FILE * file;)

Player p[4];
int ap;

double factor;
int delay;

World world[2];

bool matchexit=false;
bool gameexit;


void troxexit(void)
{
    int i;
    printf("\nFinal score:\n");
    for (i=0; i<ap; i++) {
	printf("Player %d (",i);
	if (i==0) {printf("red)   ");}
	if (i==1) {printf("green) " );}
	if (i==2) {printf("yellow)");}
	if (i==3) {printf("orange)");}
	printf(":%3d\n", p[i].score);
    }
    printf("\n\nThank you for playing TROX 2.1 beta \n\n");
    printf("If you enjoy playing TROX visit my homepage:\n");
    printf(" http://www.eikon.tum.de/~archi\n\n");
    DEBUG(fclose(file);)
}


void troxinit(void)
{
    printf("Have fun!\n\n");
    DEBUG(if ((file=fopen("./trox.log","w+"))==NULL) {perror("Error opening debug-file");exit(1);})
    atexit(troxexit);
}


void resetgame(void)
{
    int i;
    Vector ay,az;
    ay=vect(1,0,0);
    az=vect(0,0,1);
    Tripel r;
    world[1].y=vect(0,1,0);
    world[1].z=vect(0,0,1);
    r.wy=0; r.wz=0;
    for (i=0; i<ap; i++) {
	p[i].alive=true;
	if (p[i].local) {
	    r.wx=2.0*M_PI/(double)ap*(double)i;
	    world[p[i].side].y=rotate(ay,r);
	    world[p[i].side].z=rotate(az,r);
	    world[p[i].side].dx=1;
	    world[p[i].side].dy=0;
	    world[p[i].side].flip=false;
	}
    }
    createpalette();
    clearlist();
    clearimage();
    clearstatusarea();
    displaystatus();
    displayimage();
}


void printscore(void)
{
    int i;
    char s[40];
    clearstatusarea();
    for (i=0; i<ap; i++) {
	if (i==0) {sprintf(s,"PLAYER 1(RED)   : %3d",p[i].score);}
	if (i==1) {sprintf(s,"PLAYER 2(GREEN) : %3d",p[i].score);}
	if (i==2) {sprintf(s,"PLAYER 3(YELLOW): %3d",p[i].score);}
	if (i==3) {sprintf(s,"PLAYER 4(ORANGE): %3d",p[i].score);}
	outtextxy(50,i*32,s);
    }
    displaystatus();
}


void struggle(int turn)
{
    int i,c,a, sp;
    bool col;
    Vector n;
    World * w;
    if (p[turn].local) {
	w=(&world[p[turn].side]);
	if ((*w).flip) {
	    if ((*w).dx==0) {
		(*w).y=rotate((*w).y,trip(0,M_PI,0));
		(*w).z=rotate((*w).z,trip(0,M_PI,0));
	    }
	    else {
		(*w).y=rotate((*w).y,trip(M_PI,0,0));
		(*w).z=rotate((*w).z,trip(M_PI,0,0));
	    }
	    (*w).flip=false;
	}
	else {
	    (*w).y=rotate((*w).y,trip((*w).dx*factor,(*w).dy*factor,0));
	    (*w).z=rotate((*w).z,trip((*w).dx*factor,(*w).dy*factor,0));
	}
	n=rerotate(vect(0,0,BIGBALLRADIUS),whereis(*w));
	DEBUG(fprintf(file,"Sending: player:%d %d %d %f %f %f %d\n",
		      turn,turn,-1,n.x,n.y,n.z,iscollision(n));)
	sendmessage(turn,-1,n,iscollision(n));
    }
    // Ignore non-valid packets!
    do {getmessage(sp,a,n,col,true);}
    while (a>=0);
    DEBUG(fprintf(file,"Reading: player:%d %d %d %f %f %f %d\n",
		  turn,sp,a,n.x,n.y,n.z,col);)
    if (a==-2) {matchexit=true; gameexit=true; getmessage(sp,a,n,col,true);}
    else {
	if (sp!=turn) {
	    printf("Error in communication! Player:%d %d %d %f %f %f %d\n",
		   turn,sp,a,n.x,n.y,n.z,col);
	    exit(1);
	}
    }
    addtolist(n,turn); 
    if (col) {
	p[turn].alive=false;
	c=0;
	for (i=0; i<ap; i++) {
	    if (p[i].alive) {
		c++; 
		p[i].score++;
	    }
	}
	if (c<=1) {gameexit=true;}
	printscore();
	setblack(turn);
    }
} 


void resync(void)
{
    int i;
    char s[40];
    bool synced=false;
    bool ready[ap];
    bool start=true;
    int sp, a;
    for (i=0; i<ap; i++) {ready[i]=false;}
    vga_clear();
    while (!synced) {
	clearstatusarea();
	if (getshortmessage(sp,a,false)) {
	    if (a==-5) {
		if (sp==0)  {fadeout();printf("trox: Aborted by user!\n"); exit(0);}
		ready[sp-1]=true;
	    }
	}
	for (i=0; i<ap; i++) {
	    sprintf(s,"PLAYER %d : ",i+1);
	    if (ready[i]) {strcat(s,"READY");}
	    else {
		if (p[i].local) {
		    if (p[i].side==0) {strcat(s,"PRESS <TAB>");}
		    else {strcat(s,"PRESS <RETURN>");}
		    if (world[p[i].side].flip) {
			sendshortmessage(i+1,-5);
			world[p[i].side].flip=false;
		    }
		}
		else {strcat(s,"NOT READY");}
	    }
	    outtextxy(32,i*32,s);
	    if (start) {start=false; greyin();}
	}
    	displaystatus();
	if (escpressed()) {sendshortmessage(0,-5);}
	resettimer();
	synced=true;
	for (i=0; i<ap; i++) {if (!ready[i]) {synced=false;}}
    }
    greyout();
}


int main(char argc, char * argv[])
{
    int turn;
    vga_init();
    troxinit();
    initnet(argc,argv,ap,p);
    initengine();
    intro();
    initgyro();
    factor=(double)getsmallballradius()/(double)BIGBALLRADIUS*2.0;
    delay=getsmallballradius()/3;
    inittimer(DELAY,world);
    fadein();
    while (!matchexit) {
	gameexit=false;
	resync();
	resetgame();
	printscore(); 
	waituntil(10);
	greyin();
	resettimer();
	if (p[0].local) {waituntil(20);}
	resettimer();
	while (!gameexit) {
	    for (turn=0; (turn<ap)&&(!gameexit); turn++) {
		if (p[turn].alive) {
		    if (escpressed()) {sendmessage(0,-2,vect(0,0,0),false);}
		    struggle(turn);
		    clearimage();
		    turnanddraw(whereis(world[0]),0);
		    turnanddraw(whereis(world[1]),1);
		    waituntil(delay);
		    resettimer();
		    displayimage();
		    showpal();
		}
		// else {sink(turn);} if this is enabled, the grey balls will sink slowly
		// into the big ball, so you can drive over them after a while.
		// (very difficult to handle!)
	    }
	}
	resettimer();
	waituntil(10);
	greyout();
    }
    fadeout();
    return 0;
}

